package MyApp::Memcache;

use v5.30;
use Mojo::Base -base, -signatures;
use Cache::Memcached::Fast;
use IO::Compress::Gzip;
use IO::Uncompress::Gunzip;
use Storable;

has 'server';
has 'memd' => sub ($self) {
  Cache::Memcached::Fast->new({
    namespace => 'factoid_viewer',
    connect_timeout => 0.2,
    io_timeout => 0.5,
    close_on_error => 1,
    compress_threshold => 1_000,
    compress_ratio => 0.9,
    compress_methods => [ \&IO::Compress::Gzip::gzip,
                          \&IO::Uncompress::Gunzip::gunzip ],
    max_failures => 3,
    failure_timeout => 2,
    ketama_points => 150,
    nowait => 1,
    hash_namespace => 1,
    serialize_methods => [ \&Storable::freeze, \&Storable::thaw ],
    utf8 => 1,
    max_size => 512 * 1024,
    servers => [{address => $self->server}],
  })
};

sub get($self, $key) {
  $self->memd->get($key);
}

sub set($self, $key, $value) {
  $self->memd->set($key, $value, 600);
}


1;